/* config.h.  Generated by configure.  */
/* config.h.in.  Generated from configure.in by autoheader.  */

/* Build MJPEG software playback */
#define BUILD_MJPEG 1

/* Define to 1 if you have the <alloca.h> header file. */
#define HAVE_ALLOCA_H 1

/* Inline PPC Altivec primitives available */
/* #undef HAVE_ALTIVEC */

/* Inline GCC MMX assembler supported in C compiler */
#define HAVE_ASM_MMX 1

/* nasm assembler is available */
#define HAVE_ASM_NASM 1

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define to 1 if you have the <fenv.h> header file. */
#define HAVE_FENV_H 1

/* fmax support */
/* #undef HAVE_FMAX */

/* Define to 1 if you have the <getopt.h> header file. */
#define HAVE_GETOPT_H 1

/* getopt support */
#define HAVE_GETOPT_LONG 1

/* Gtk library present */
#define HAVE_GTK 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* dlopen support */
#define HAVE_LIBDL 1

/* libdv is present */
/* #undef HAVE_LIBDV */

/* Posix pththreads lib present */
#define HAVE_LIBPTHREAD 1

/* libquicktime is present */
/* #undef HAVE_LIBQUICKTIME */

/* libxf86dga is present */
#define HAVE_LIBXXF86DGA 1

/* lround support */
#define HAVE_LROUND 1

/* Define to 1 if you have the <malloc.h> header file. */
#define HAVE_MALLOC_H 1

/* memalign support */
#define HAVE_MEMALIGN 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* posix_memalign support */
#define HAVE_POSIX_MEMALIGN 1

/* Compiling for PowerPC CPU */
/* #undef HAVE_PPCCPU */

/* pthread stack sizes accesible */
/* #undef HAVE_PTHREADSTACKSIZE */

/* SDL library present */
#define HAVE_SDL 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/soundcard.h> header file. */
#define HAVE_SYS_SOUNDCARD_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Building for Linux - using the video4linux API */
#define HAVE_V4L 1

/* Whether XFree86 is available */
#define HAVE_X 1

/* Compiling for x86 architecture CPU */
#define HAVE_X86CPU 1

/* Is __progname defined by system? */
#define HAVE___PROGNAME 

/* YV12 support in libdv */
/* #undef LIBDV_PAL_YV12 */

/* Building mjpegtools */
#define MJPEGTOOLS 1

/* Optimised no DEBUG build, assertions disabled */
/* #undef NDEBUG */

/* Name of package */
#define PACKAGE "mjpegtools"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME ""

/* Define to the full name and version of this package. */
#define PACKAGE_STRING ""

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME ""

/* Define to the version of this package. */
#define PACKAGE_VERSION ""

/* Format string for 64 bit integers */
#define PRID64_STRING_FORMAT "lld"

/* Define as the return type of signal handlers (`int' or `void'). */
#define RETSIGTYPE void

/* The size of a `int', as computed by sizeof. */
#define SIZEOF_INT 4

/* The size of a `long', as computed by sizeof. */
#define SIZEOF_LONG 4

/* The size of a `long long', as computed by sizeof. */
#define SIZEOF_LONG_LONG 8

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Version of this build */
#define VERSION "1.6.1.93"

/* Define to 1 if your processor stores words with the most significant byte
   first (like Motorola and SPARC, unlike Intel and VAX). */
/* #undef WORDS_BIGENDIAN */

/* Define to 1 if the X Window System is missing or not being used. */
/* #undef X_DISPLAY_MISSING */

/* Number of bits in a file offset, on hosts where this is settable. */
#define _FILE_OFFSET_BITS 64

/* Enable GNU extensions on systems that have them.  */
#ifndef _GNU_SOURCE
# define _GNU_SOURCE 1
#endif

/* Define for large files, on AIX-style hosts. */
/* #undef _LARGE_FILES */

/* Define to empty if `const' does not conform to ANSI C. */
/* #undef const */

/* Define as `__inline' if that's what the C compiler calls it, or to nothing
   if it is not supported. */
/* #undef inline */
